/*
 * $Id: g_diff.c,v 1.7 2002/08/13 11:44:23 oliver Exp $
 * $Log: g_diff.c,v $
 * Revision 1.7  2002/08/13 11:44:23  oliver
 * Merged in userdefined slicetab
 *
 * Revision 1.6.2.3  2002/02/12 17:19:39  oliver
 * - fixed \S typo
 * - replaced another fprintf with msg
 *
 * Revision 1.6.2.2  2002/01/23 14:42:42  oliver
 * General:
 * * expanded the usage message and also put an example slicetab.xvg in
 *   there
 * * replaced all (?) fprintf(stderr,...) with msg() or dmsg() from
 *   utilgmx.c
 * * changed a few dmsg() into dfprintf() because they clutter stderr
 *   when debugging
 * * new function read_userslicetab()
 *
 * read_userslicetab():
 *   Reads in a one-line file (+comments) which defines the top
 *   boundaries of the slices. The last entry is the box width Bmax in this
 *   dimension. It converts them into relative slice thicknesses:
 * 	relative = entry/Bmax
 *   If in debug mode, the whole userslicetab is printed out.
 *
 * find_slice():
 * * corrected calculation of relative slice thicknesses (the one before
 *   was total bu||0(%$ !) -- now done in read_slicetab()
 * * removed one superfluous sanity check
 * * introduced DEBUG2 for parts of the code which generate MASSIVE debug
 *   files
 *
 * diffusion_plot():
 * * also uses userslicetab
 * * change in output format:
 *   - plots the center of the slice as x value and D and sigma(D) as y
 *     and dy
 *   - changed nxy style output to xmgrace style: data sets are separated
 *     by a '&\n'
 *
 * Revision 1.6.2.1  2002/01/21 22:36:54  oliver
 * g_diff now allows to read in a userdefined slicetab. This is still
 * experimental and therefore only on the branch
 * rel-0-1-userdefined-slicetab.
 *
 * See g_diff -h for an explanation.
 *
 * The stable version of g_diff is still on the main branch.
 *
 * Revision 1.6  2002/01/16 10:51:30  oliver
 * * Reverted to original behaviour by considering all slices which
 *   contain at least _1_ molecule (slCount[ii][i] > 0); this was set to
 *   slCount > 1 in the last version. Now I am setting the weight before
 *   the LSQ fit to 1 if there is only one molecule to add to the msd(t)
 *   fit. Typically, the program barffs on these cases nevertheless :-(
 *   (This I consider a BUG).
 * * added bug section
 * * Changed debugging output
 *   - uses now regular -debug features instead of the 'bug' file and
 *     various fprintfs(stderr,...)
 *   - uses debugg msg functions from utilgmx.c:
 *     msg, dmsg, dfprintf
 *
 * Revision 1.5  2002/01/14 10:11:05  oliver
 * * added function names to debug output
 * * changed debug output of LSQ weights
 * * BUG: if there is only one molecule for the LSQ the program barffs
 *   NaN and such. Reason: the weights that enter are the inverse of the
 *   variance which vanishes for one observation.
 *   Fixing it by substituting '1' for the weight did not work
 *
 * Revision 1.4  2002/01/10 15:19:54  oliver
 * - g_diff seems to work with Gromacs3
 * - moved sources from the g_count module to the g_diff module
 *
 * Revision 1.3  2001/11/12 23:05:43  oliver
 * * added SRCID string which contains the CVS identifier
 * * Version is displayed after the copyright
 *
 *
 */
static char *SRCID_g_diff_c = "$Id: g_diff.c,v 1.7 2002/08/13 11:44:23 oliver Exp $";

/* GMX_VERSION is currently not used in g_diff.c */
#ifndef GMX_VERSION
#define GMX_VERSION 3
#endif

#include <math.h>
#include "sysstuff.h"
#include "string.h"
#include "typedefs.h"
#include "smalloc.h"
#include "macros.h"
#include "rmpbc.h"
#include "vec.h"
#include "xvgr.h"
#include "pbc.h"
#include "copyrite.h"
#include "futil.h"
#include "statutil.h"
#include "rdgroup.h"
#include "utilgmx.h"

/****************************************************************************/
/* This program calculates the diffusion coefficients of water per slice, as*/
/* function of the z-coordinate, from the MSD displacements using the       */
/* Einstein relation                                                        */
/*                                                                          */
/* Peter Tieleman, January 1996                                             */
/****************************************************************************/

#define TRESH 0.2    /* percentage of msd graph to neglect for fitting */
#define NRESTART 5  /* restart every 5 times per 5 ps */
#define LOWEST 20    /* lowert nr of waters in slice that stilll counts */

static int axis = 2; /* use Z-axis as default. */
static matrix box;   /* box (3x3)                               */


/* lsq things stolen from g_msd.cc. */
typedef struct {
  double yx,xx,sx,sy;
  double sigw_y;     /* sum of the weight factors */
  int    np;
} t_lsq;

void init_lsq(t_lsq *lsq)
{
  lsq->yx=lsq->xx=lsq->sx=lsq->sy=0.0;
  lsq->sigw_y=0.0;
  lsq->np=0;
}

void add_lsq(t_lsq *lsq,real x,real y,real w_y)
{
#ifdef DEBUG
  dfprintf("add_lsq(): Adding x,y,w_y: %f %f %f\n",x,y,w_y);
#endif
  lsq->yx+=w_y*y*x;
  lsq->xx+=x*x*w_y;
  lsq->sx+=x*w_y;
  lsq->sy+=w_y*y;
  lsq->np++;
  lsq->sigw_y+=w_y;
}

void get_lsq_ab(t_lsq *lsq,real *a,real *b)
{
  real yx,xx,sx,sy;
 
  yx=lsq->yx/lsq->sigw_y;
  xx=lsq->xx/lsq->sigw_y;
  sx=lsq->sx/lsq->sigw_y;
  sy=lsq->sy/lsq->sigw_y;

#ifdef DEBUG
  dfprintf("np=%d   sigw_y=%f\n",lsq->np, lsq->sigw_y);
  dfprintf("average yx,xx,sx,sy:%f %f %f %f\n",yx,xx,sx,sy);
#endif
  (*a)=(yx-sx*sy)/(xx-sx*sx);
  (*b)=(sy)-(*a)*(sx);
}

int find_slice(rvec cm, int nslices, real **userslicetab)
{
  int i, slice;
  rvec tmp;
  
  for (i = 0; i < DIM; i++)
    tmp[i] = cm[i];

  while (tmp[axis] < 0)
    tmp[axis] += box[axis][axis];
  while (tmp[axis] > box[axis][axis])
    tmp[axis] -= box[axis][axis];

  if (userslicetab) {
    for (slice=0; slice < nslices && 
	   tmp[axis] > userslicetab[slice][0] * box[axis][axis];
	 slice++)
#if DEBUG2
      /* this dfprintf  produces HUGE debug files. Use with care */
      {
	dfprintf(
	 "slice %d  tmp[axis]=%f > %f=userslicetab[slice][0] * box[axis][axis]\n",
	 slice, tmp[axis], userslicetab[slice][0] * box[axis][axis]);
      }
#endif 
	;
    /* sanity check */
    if (tmp[axis] < 0) {
      slice = -1;
    } 
  } else {
  /* cannot find trunc() anywhere...
    slice = trunc(tmp[axis] * nslices / box[axis][axis] ); 
  */
  slice = (int) (tmp[axis] * nslices / box[axis][axis] ); 
  }

  /* not sure why Peter(?) hacked this in; somehow it makes the next if
     statement's second condition obsolete 
  */
  if (slice == nslices) {
    dmsg("Putting slice with max. number in 0\n");
    return 0;
  }

  if (slice < 0 || slice >= nslices)
  {
    msg("Coordinate: %f\n", cm[axis]);
    msg("HELP PANIC! slice = %d, OUT OF RANGE!\n",slice);
    slice = -1;
  }
  return slice;
}

void make_slicetab(rvec x0[], atom_id *index, int ngx, 
		   int nslices, real **userslicetab,
		   int *slCount, int *slTab)
{
  int i; 
  int slice;

  for (i = 0; i < nslices; i++)
    slCount[i] = 0;

  for (i = 0; i < ngx; i++) /* put all atoms in index file in slice */
  {
    slice = find_slice(x0[index[i]], nslices, userslicetab);
    if (slice != -1)
    {
      slTab[i] = slice;
      slCount[slice]++;
    }
    else
      msg("Warning: find_slice returned -1\n");
  }

#ifdef DEBUG
  for (i = 0; i < nslices; i++)
    dfprintf("make_slicetab(): %5d molecules in slice %3d\n", slCount[i], i); 
#endif
}




void calc_diffusion(char *fn, atom_id *index, int ngx, real interval, 
		    int nslices, real **userslicetab,
		    rvec **diff, rvec **stdev, rvec **cdiff, 
		    int **count)
{
  rvec *x0,          /* coordinate frame                        */
    *xprev, hbox,    /* previous frame                          */
    **ref;           /* reference frame                         */
  int   status;
  rvec dr,r2,        /* displacement, squared displacement      */
      **slGframe,    /* per slic, diffusion in three directions */
      **slSigGframe, /* sum of slGframe squared                 */
      **slCframe,    /* collective diffusion                    */
      *slD,*slD2,    /* average dif. co.per slice, square of that*/
      *slCD;         /* average dif. co. of a whole slice       */
  real t, tin[NRESTART]; /* time from trajectory, interval time */
  int natoms,        /* nr. atoms in trj                        */
      i,ii,m,teller = 0,
     **slTab,        /* slTab[restart][atom] is the slice atom is in*/
     **slCount,      /* slCount[restart][slice] is the nr. of atoms in slice */
    *slDcount;  
  t_lsq ***slTot;   /* sucks. slTot[restart][slice][dimensin] */
  t_lsq ***slCTot;  /* sucks too. slCTot[restart][slice][dim] */
  real a, b;
  real weight;


  if ((natoms = read_first_x(&status,fn,&t,&x0,box)) == 0) 
  {
    msg("Could not read coordinates from statusfile\n");
    exit(1);
  }

  msg("\nBox divided in %d slices along axis %c.\n", nslices,axis + 'X');
  if (userslicetab) {
    msg("Slices are calculated from user supplied slicetab.\n");
  } else {  
    msg("Initial width of slice: %f\n",box[axis][axis]/nslices);
  };
  for (i = 0; i < NRESTART; i++)
  {
    tin[i] = t+i*NRESTART/interval; /* set initial restart times */
  }

  /* initialize tons of memory */
  snew(xprev, natoms);
  snew(ref, NRESTART);
  snew(slGframe, NRESTART);
  snew(slCount, NRESTART);
  snew(slTot, NRESTART);
  snew(slTab, NRESTART);
  snew(slSigGframe, NRESTART);
  snew(slCframe, NRESTART);
  snew(slCTot, NRESTART);

  for (i = 0; i < NRESTART; i++)
  {
    snew(ref[i], natoms);
    snew(slCount[i], nslices);
    snew(slTot[i], nslices);
    snew(slCTot[i], nslices);  /* collective diffusion */
    snew(slTab[i], natoms);
    snew(slGframe[i], nslices);
    snew(slCframe[i], nslices); /* collective diffusion */
    snew(slSigGframe[i], nslices);
    for (m = 0; m < nslices; m++)
      {
	snew(slTot[i][m], DIM);
	snew(slCTot[i][m], DIM);
      }
  }
  snew(slD, nslices);
  snew(slCD, nslices);
  snew(slD2, nslices);
  snew(slDcount, nslices);
  snew(*diff, nslices); 
  snew(*stdev, nslices);
  snew (*cdiff, nslices);

  for (ii = 0; ii < NRESTART; ii++)
  {
    memcpy(ref[ii],x0,natoms*sizeof(rvec)); /* this is the reference frame */
    for (i = 0; i < nslices; i++)
    {
      for (m = 0; m < DIM; m++)
      {
	init_lsq(&(slTot[ii][i][m]));
	init_lsq(&(slCTot[ii][i][m]));
      }
    }
  }

  memcpy(xprev,x0,natoms*sizeof(rvec)); /* save this is the previous frame */

  teller = 0; 

  /* make initial slicetables. This is a problem, because the table
     for restart 5 will be updated only after 10 ps. After the first
     time they'll be updated every 5 ps 
   */
  for (ii = 0; ii < NRESTART; ii++) 
      make_slicetab(x0, index, ngx, 
		    nslices, userslicetab, slCount[ii], slTab[ii]);

  /*********** Start processing trajectory ***********/
  do 
  {
    /*
    if ((teller++ % 10) == 0)
      msg("\nFrame: %d\n",teller-1); 
    */
    
    for (i = 0; i < nslices; i++)
      for (ii = 0; ii < NRESTART; ii++)
	for (m = 0; m < DIM; m++)
	{
	  slGframe[ii][i][m] = 0;
	  slCframe[ii][i][m] = 0;
	  slSigGframe[ii][i][m] = 0;
	}
    
    for(m=0; (m<DIM); m++) 
      hbox[m]=0.5*box[m][m];
    
    /* prepare data so pbc is gone */
    for (i = 0; i < natoms; i++)  
    {
      for(m=0; (m<DIM); m++) 
      {
	while(x0[i][m]-xprev[i][m] <= hbox[m])
	  x0[i][m] += box[m][m];
	while(x0[i][m]-xprev[i][m] >  hbox[m])
	  x0[i][m] -= box[m][m];
      }
    }

    /* Loop over all atoms in the group (ngx), so typically all oxygen
       atoms. Calculate the displacement of each oxygen with respect to the
       previous frame, square it, and add it to the total msd for the 
       slice the atom belongs in. The index for the slice atom i is in 
       is just slTab[i], so slGframe[slTab[i]] is the msd for the atoms 
       in slice slTab[i], for one frame 
    */
    for (ii = 0; ii < NRESTART; ii++)
    {
      for (i = 0; i < ngx; i++) 
      {
	for (m = 0; m < DIM; m++)
	{
	  dr[m] = x0[index[i]][m] - ref[ii][index[i]][m];
	  r2[m] = dr[m]*dr[m];
          slCframe[ii][slTab[ii][i]][m] += dr[m];
	  slGframe[ii][slTab[ii][i]][m] += r2[m];
	  slSigGframe[ii][slTab[ii][i]][m] += r2[m]*r2[m];
	}
	/* ok, this sucks too. slGframe[ii] is the array with the r2
	 *for all _slices_, in all three dimensions, for restart point
	 *ii. slTab[ii][i] is the slice atom i is in in restart point
	 *ii, and [m] is the dimension we're looking at.  
	 */
	
      }

      /* now we have the sum of the displacements for the collective 
	 version but still need to square it. */
      for (i=0;i<nslices;i++)
	for (m=0;m<DIM;m++)
	  slCframe[ii][i][m] *= slCframe[ii][i][m];

      
#ifdef DEBUG
      dfprintf("calc_diffusion(): Calculated displacements for frame %d, "
	       "t_restart %d\n", teller, ii);
#endif
      /* now we have calculated the msd for all atoms in this frame.
       Check if for certain starting points we have reach 'interval',
       and add the diffusion coeffecient to a bin (slD[i])
       */
      if ((t-tin[ii]) > TRESH*interval)  
      {
	for (i = 0; i < nslices; i++)
	  if (slCount[ii][i]) {
	    for (m = 0; m < DIM; m++)
	      {
		slSigGframe[ii][i][m] /= slCount[ii][i];
		slGframe[ii][i][m] /= slCount[ii][i];
		slCframe[ii][i][m] /= slCount[ii][i]; 
		/* if there is only one molecule in this slice 
		   we need to adjust the weight to 1 

		   Not true (??) -- perhaps we cannot do the LSQ properly
		   with only one observation?
		   Therefore, we throw out N<2 cases (see above)

		   ARGH. Still not workin (despite slCount > 1). Do I 
		   have to zero sl... properly?

		weight = slSigGframe[ii][i][m] - 
		         slGframe[ii][i][m]*slGframe[ii][i][m];

		*/

		weight = (slCount[ii][i] == 1 ? 1 :
		     slSigGframe[ii][i][m] - 
                     slGframe[ii][i][m]*slGframe[ii][i][m] );



#ifdef DEBUG
		dfprintf("t_i=%2d sl=%2d N=%4d dim=%1d  1/weight=%f\n",
		    ii,i,slCount[ii][i],m, 1/weight);
		if (slCount[ii][i] == 1) {
		  dmsg("*** Warning: only ONE molecules in slice %d (restart=%d)\n",
		       i, ii);
		};
#endif

		add_lsq(&(slTot[ii][i][m]), t-tin[ii], 
			slGframe[ii][i][m],1/weight); 
		add_lsq(&(slCTot[ii][i][m]), t-tin[ii],
			slCframe[ii][i][m],1);
	      }
	  }
#ifdef DEBUG	   
	dfprintf("\n");
#endif
      }

      if ( (t - tin[ii]) > interval )
      {
	tin[ii] = t; 
	for (i = 0; i < nslices; i++)
	  if (slCount[ii][i])
	  {
	    /* get diffusion coeffecient, and zero the lsq for next time */
	    for (m = 0; m < DIM; m++)
	    {
	      get_lsq_ab(&(slTot[ii][i][m]),&a,&b);
	      slD[i][m] += a*500; 
	      slD2[i][m] += a*a*500*500;  /* keep square */
	      init_lsq(&(slTot[ii][i][m]));
	      get_lsq_ab(&(slCTot[ii][i][m]), &a, &b);
	      slCD[i][m] += a*500;
	      init_lsq(&(slCTot[ii][i][m]));
	    }
	    slDcount[i]++;
	  }
	/* now make new slTab and new ref. frame for this ii */
	memcpy(ref[ii],x0,natoms*sizeof(rvec));
	make_slicetab(x0, index, ngx, nslices, userslicetab,
		      slCount[ii], slTab[ii]);
#ifdef DEBUG
	dmsg("Making sltab for restart %d, time %f, frame%d\n",
		ii,t,teller);
	dmsg("tin[%d] = %f\t",ii,tin[ii]);
#endif
      }
    }   /* end loop over NRESTARTS */
    memcpy(xprev,x0,natoms*sizeof(rvec)); /* current frame becomes previous */
    
  } while (read_next_x(status,&t,natoms,x0,box));
  /*********** done with status file **********/
  
  for (i = 0; i < nslices; i++)
    for (m = 0; m < DIM; m++)
      {
	if (slDcount[i] == 0)
	  msg("No atoms in slice %d,skipping.\n",i);
	else
	{
	  slD[i][m] /= slDcount[i];
	  slCD[i][m] /= slDcount[i];
	  slD2[i][m] /= slDcount[i];
	}
	slD2[i][m] -= slD[i][m]*slD[i][m];  /* get variance */
      }
  
  *stdev = slD2;
  *diff = slD;
  *cdiff = slCD;
  *count = slDcount;
  
  close_trj(status);
}

void diffusion_plot(rvec diffusion[], rvec variance[], rvec cdiff[], 
		    int count[], char *afile, char *bfile, 
		    int nslices, real **userslicetab)
{
  FILE       *dif,*cdif;
  int        slice, d;     
  char       buf[256];               /* for xvgr title */
  real       center;                 /* center of a slice */
  real       *boundaries = NULL;     /* boundaries of slices */

  sprintf(buf,"Diffusion per slice");
  dif = xvgropen(afile,buf,"z (nm)","D (nm\\S2\\N ns\\S-1\\N)");
  sprintf(buf,"Collective diffusion of slices");
  cdif = xvgropen(bfile,buf,"z (nm)","D (nm\\S2\\N ns\\S-1\\N)");

  /* Write the centers of the slices to the output. For userdefined
   slicetab the calculation is simplified if we put '0' in the first
   position of slicetab:

         center_i = (z_i + z_(i-1))/2

   There is certainly a more elegant way to do it but this works well enough... 
  */

  if (userslicetab) {
    snew(boundaries, nslices+1);
    boundaries[0] = 0;
    for (slice=0; slice < nslices; slice++) {
      boundaries[slice+1] = userslicetab[slice][0];
    }
  }

  for (d=XX; d < DIM; d++) {
    for (slice = 0; slice < nslices; slice++)
      {
	center = (userslicetab) ? 
	  (boundaries[slice] + boundaries[slice+1])/2 * box[axis][axis] :
	  (2.0*slice + 1.0)/(2.0*nslices) * box[axis][axis];

	if (count[slice]) {
	  fprintf(dif,"%12g     %12g %12g\n", center,
		  diffusion[slice][d], sqrt(variance[slice][d])/sqrt(count[slice]));
	}
	fprintf(cdif,"%12g     %12g\n", center, cdiff[slice][d]);
      }
    fprintf(dif,  "&\n");
    fprintf(cdif, "&\n");
  }


  /* produces nxy format files; suitable for xmgr but no fun with xmgrace */
  /*
  for (slice = 0; slice < nslices; slice++)
  {
    center = (userslicetab) ? 
      (boundaries[slice] + boundaries[slice+1])/2 * box[axis][axis] :
      (2.0*slice + 1.0)/(2.0*nslices) * box[axis][axis];

    if (count[slice])
      fprintf(dif,"%12g     %12g %12g %12g %12g %12g %12g\n", 
	      center,
	      diffusion[slice][XX], sqrt(variance[slice][XX])/sqrt(count[slice]),
	      diffusion[slice][YY], sqrt(variance[slice][YY])/sqrt(count[slice]),
	      diffusion[slice][ZZ], sqrt(variance[slice][ZZ])/sqrt(count[slice]));
    fprintf(cdif,"%12g     %12g %12g %12g\n", 
	    center, 
	    cdiff[slice][XX], cdiff[slice][YY], cdiff[slice][ZZ]);
  }
  */
}

int read_userslicetab (char *fnm, int *nslices, real ***userslicetab) {
  /* read in user supplied slice tab and convert into a list of
     relative slice boundaries 
  */
  int       nx;             /* number of lines of data in slicetab.xvg */
  int       i;

  msg("Using user defined slicetab from file %s\n\n", fnm);

  nx = read_xvg(fnm, userslicetab, nslices);

  if (nx > 1) {
    msg("Reading slicetab: more than one line of input "
	"(N=%d.\nOnly the first line will be considered.\n", nx);
  } else if (nx < 1) {
    msg("Reading slicetab: no data.\n");
    fatal_error(0,"Program %s halted",Program());
  } 

  if (*nslices < 1) {
    msg("Reading slicetab: no slices defined (nslices=%d).\n", *nslices);
    fatal_error(0,"Program %s halted",Program());
  }

  if (bDebugMode()) {
    msg("Contents of user slicetab %s:\n\n", fnm);
    for(i=0; i < *nslices; i++) {
      msg("%f ", (*userslicetab)[i][0]);
    }
    msg("\n");
  }

  /* transform into list of relative thicknesses */
  for (i=0; i < *nslices; i++) {
    (*userslicetab)[i][0] = (*userslicetab)[i][0]/(*userslicetab)[(*nslices)-1][0];
  }

  if (bDebugMode()) {
    msg("Contents of user slicetab after normalising to the last element \n"
	"(the original box size along the chosen axis):\n");
    for(i=0; i < *nslices; i++) {
      msg("%f ", (*userslicetab)[i][0]);
    }
    msg("\n");
  }

  return nx;
}

int main(int argc,char *argv[])
{
  static char *desc[] = {
    "The program calculates the diffusion coefficient of a species given in "
    "the index file within slices of the simulation box. It uses the Einstein "
    "relation and a least square fit to the last 80% of the MSD(t).",
    "It uses 5 time origins (compiled in, 'NRESTART') and calculates D after the "
    "time given with -t.",
    "[PAR]Normally, the program slices the box into N slices (-sl N) at each "
    "time step. With -T slicetab.xvg a user defined slice tab can be "
    "provided. The format is one line of space-separated floats. Comments "
    "are indicated by a hash (#) at the beginning of the line. g_diff "
    "calculates relative slice thicknesses from the numbers given and "
    "slices the box at each time step accordingly.",
    "[PAR]EXAMPLE slicetab.xvg[PAR]"
    "# example slice tab\n"
    "# for 5 slices (along Z):\n"
    "#  1       2        3         4         5\n"
    "# bulk |  mouth |  pore  |  mouth  | (bulk) Z_box\n"
    "      1.5      1.9      2.7       3.1        4.5\n"
  };

  static char *bugs[] = {
  "Molecules are included in the calculation of D(slice) on the basis of "
  "their _starting_ position. For thin slices and longer time intervals t "
  "leads to an systematic error in D",
  "For only one molecule in a slice, the least-squares fitting procedure to "
  "the msd(t) tends to give NaN or grossly wrong (large) numbers.",
  "OUTPUT CHANGED: the diffusion coefficient is now plotted at the CENTER of "
  "a slice. It used to be the top boundary of the slice.",
  "Not sure if one can compare D's calculated from "
  "slices of different thickness."
  };

  static real  interval = 5.0;    /* interval to use            */
  static int  nslices = 1;
  static int axis = 2;

  t_pargs pa[] = {
    { "-t", FALSE, etREAL, {&interval}, 
      "Length of a time-interval."},
    { "-sl", FALSE, etINT, {&nslices},
      "Nr. of slices" },
    { "-d", FALSE, etINT, {&axis},
      "-d 0|1|2, axis (x=0, y=1, z=2)"}
  };
 
  char      **grpname;            	    /* groupnames                 */
  rvec      *diff, *variance;
  rvec      *cdiff;                         /* collective diff/slice      */
  int       *count;                        
  int       ngx;
  atom_id   **index;             	    /* indices                    */
  real      **userslicetab = NULL;          /* user defined slicetab      */
                                   /* use as userslicetab[slice_nr][0] */
  t_filenm  fnm[] = {             	    /* files for g_order 	  */
    { efTRX, "-f", NULL, ffREAD },    	    /* trajectory file 	          */
    { efNDX, NULL, NULL, ffREAD },    	    /* index file 		  */
    { efXVG,"-T",  "slicetab.xvg",  ffOPTRD}, /* user defined slicetab   */
    { efXVG,"-o",  "diffusion.xvg", ffWRITE}, /* xvgr output file 	  */
    { efXVG,"-oc", "slicediff.xvg", ffWRITE}  /* xvgr output file         */
  };
#define NFILE asize(fnm)

  CopyRight(stderr,argv[0]);

  parse_common_args(&argc,argv,PCA_CAN_TIME | PCA_CAN_VIEW,
		    NFILE,fnm,asize(pa),pa,asize(desc),desc,asize(bugs),bugs);

  fprintf (stderr, "\nVersion: %s\n\n", SRCID_g_diff_c);

  if (bDebugMode()) {
    dfprintf ("%s -- debugging...\n", Program());
    dfprintf ("Version: %s\n\n", SRCID_g_diff_c);
  }

  if (opt2bSet("-T",NFILE,fnm)) {
    read_userslicetab (opt2fn("-T",NFILE,fnm), &nslices, &userslicetab);
  } else {
    msg("Dividing box in %d slices.\n\n", nslices);    
  }
  
 

  msg("Taking %c as axis\n",'X'+axis);

  snew(grpname,1);
  snew(index,1);
  rd_index(ftp2fn(efNDX,NFILE,fnm),1,&ngx,index,grpname);    /* read index */

  calc_diffusion(ftp2fn(efTRX,NFILE,fnm), index[0], ngx, interval, 
		 nslices, userslicetab, &diff, &variance, &cdiff, &count);

  diffusion_plot(diff, variance, cdiff, count, opt2fn("-o",NFILE,fnm), 
		 opt2fn("-oc",NFILE,fnm), nslices, userslicetab);

  thanx(stdout);
  return 0;
}








