/*
 $Id: g_water_diffusion.c,v 1.5 2002/01/11 17:06:21 oliver Exp $
 g_water_diffusion  by MBU 28.10.2001
  Calculates the diffusion coefficient in xy for slices of water.
 $Log: g_water_diffusion.c,v $
 Revision 1.5  2002/01/11 17:06:21  oliver
 Deleted comment in bugs again: The program calculates <r^2> by a
 random walk approach. This appears to be valid although one probably
 needs quite a lot of molecules and steps to average about so that the
 omission of the cross terms (as in this program) is justified:

 <z^2> = <sum_i dz_i * sum_j dz_j>
       = sum_i <dz_i^2> + sum_(i != j) <dz_i dz_j>
       = sum_i <dz_i^2> = 2Dt

 (dz_i is the displacement during the i-th step, z^2 the mean square
 displacement in z direction; <...> is an ensemble average)

 Revision 1.4  2002/01/09 17:29:39  oliver
 added warning in Bugs section that this program apparently does not use
 the Einstein relation to calculate D

 Revision 1.3  2001/12/18 22:50:21  oliver
 * fixed calculation of D_xyz: D_xyz = msd / (6*dt)
   (parentheses added)
 * version string:
   - changed name to SRCID_g_water_diffusion_c
   - print current version after the standard GROMACS banner
 * added preprocessor directives which allow compilation for GROMACS
   version 2 or 3
      gcc -D GMX_VERSION=2 ....
   sets the version flag. The default is version 2 but a warning is
   printed if it is not explicitly set.
   For version 3 compilability only the arg. list of
   parse_common_args() had to be modified.
 * added bugs[] section in the help output, including the warning of
   the sensitivy of the results from t_min (found by Rich)

 Revision 1.2  2001/12/06 12:33:11  rlaw
 I've just added a comment!  Added id and log line.
  ----------------------------------------------------------------------

  Added density calculation
  Added 3D Maple graphs

 */
static char *SRCID_g_water_diffusion_c = "$Id: g_water_diffusion.c,v 1.5 2002/01/11 17:06:21 oliver Exp $";

#ifndef GMX_VERSION
#define GMX_VERSION 2
#warning *** Compiling for Gromacs version 2. ***
#warning You should specify the version of Gromacs to compile for explicitly, e.g. 
#warning as -D GMX_VERSION=2|3 
#endif


#include <math.h>
#include <string.h>
#include <stdlib.h>      /* for malloc */
#include "statutil.h"
#include "sysstuff.h"
#include "typedefs.h"
#include "smalloc.h"
#include "macros.h"
#include "vec.h"
#include "pbc.h"
#include "copyrite.h"
#include "futil.h"
#include "statutil.h"
#include "rdgroup.h"
#include "mshift.h"
#include "princ.h"
#include "rmpbc.h"
#include "xvgr.h"
#include "txtdump.h"
#include "tpxio.h"
#include "pdbio.h"
#include "confio.h"




/** VARIABLE DEFINITIONS **/
  
  struct water_struct
  {
   int    slice;
   float  nsteps;
   float  t_origin;
   float  t_current;
   float  ax;
   float  ay;
   float  az;
   float  mod_a;
   float  a2x;
   float  a2y;
   float  a2z;
   };  

  /* Slice properties, calculated every timestep and averaged */        
  struct slice_struct
  {
   int    n_diff;      /* Number of diffusion coefficients calculated in this slice  */
   float  delta_t;     /* Average time a water molecule stayed in this slice         */
   float  n_times;     /* Total number of molecules that have left the slice         */
   float  n_stay;      /* Average number of waters staying in the slice each step    */
   float  n_new;       /* Average number of waters new in the slice each step        */
   float  n_old;       /* Average number of waters that were in the slice each step  */
   float  flux_up;     /* Average flux through the upper (+ve z-axis) slice boundary */
   float  flux_down;   /* Average flux through the lower (-ve z-axis) slice boundary */
   float  density_M;   /* Average density of water molecules in this slice           */
   float  density_S;
   float  diff_xyz_M;
   float  diff_xy_M;
   float  diff_z_M;
   float  mean_free_path_M;
   float  diff_xyz_S;
   float  diff_xy_S;
   float  diff_z_S;
   float  mean_free_path_S;
   };

  /* Diffusion coefficient properties, calculated when a water leaves a slice */
  struct diff_struct
  {
   int    slice;
   float  nsteps;
   float  t_origin;   
   float  diff_z;
   float  diff_xy;
   float  diff_xyz;
   float  mean_ax; 
   float  mean_ay; 
   float  mean_az; 
   float  mean_mod_a; 
   float  mean_a2x;
   float  mean_a2y;
   float  mean_a2z;
   }; 

  struct temp_struct
  {
   int    n_stay; 
   int    n_new;  
   int    n_old;  
   int    n_up;
   int    n_down;
   float  atoms;
   float  mass;
   float  charge;
  };


/*** GLOBAL VARIABLES ***/

FILE *debugfile;               /* Debug file is global    */


/*** FUNCTIONS ***/
char ** allocate_matrix(int length_of_element, int number_of_elements);

void get_mean_free_path(rvec *x_new, rvec *x_old, int index, float *Box, float *d);

FILE *xvgropen_nxy(char *fn,char *title,char *xaxis,char *yaxis);

FILE *xvgropen_xydy(char *fn,char *title,char *xaxis,char *yaxis);

FILE *create_maple_output(char *filename);
FILE *advance_maple_output(FILE *f_out, float xmin, float dx, float y, int N, struct temp_struct *temp, float K );

void close_maple_output(FILE *f_out);




/*** FUNCTION MAIN ***/

int main(int argc,char *argv[])
{
  static char *desc[] = {
    "g_water_diffusion (version by MBU 22.10.2001) computes the diffusion",
    "coefficients (in nm*nm/ps) using the Einstein relation. It makes",
    "slices of water to calculate the diffusion coefficient as a function of z."
  };
  
  static int freq=1, n_slices=30, t_min=25, No_of_diff=5000, maple=10;  
  t_pargs pa[] = {
    { "-skip",  FALSE, etINT, {&freq}, "Only write every n-th frame to matrix." },
    { "-sl",    FALSE, etINT, {&n_slices}, "Number of slices along the z-axis." },
    { "-tmin",  FALSE, etINT, {&t_min}, "Minimal 'time' (number of frames!!!)  a water has to spend in a slice." },
    { "-mfreq", FALSE, etINT, {&maple}, "Only write out every n-th frame to 3D maple plot." },
    { "-ndiff", FALSE, etINT, {&No_of_diff}, "Number of diffusion coefficients to calculate." }

  };

        
  /* Gromacs variables */  
  
  FILE       *out_box, *out_mean, *out_numb, *out_diff1, *out_diff2, *out_diff3;
  FILE       *out_dens, *out_flux, *out_sum, *out_time;
  FILE       *out_maple;

  char       *lega[] = { "Flux UP", "Flux DOWN" };
  char       *legb[] = { "Staying", "Leaving", "New", "Total" };


  int        status;
  t_topology top;
  rvec       xcm;       /* Centre of mass */
  real       tm;
  rvec       *x_new, *x_old;
  matrix     box;
  real       t, Value;
  int        natoms;
  char       *grpname, title[256];
  int        gnx;
  atom_id    *index;

  
  /* Loop variables */
  float      *q, *m;
  char       **atom_name, **residue_name;
  int        *residue_number, *atom_number; 
  float      area, area_tmp, Box[3];
    
  
  /* Diffusion Variables (MBU) */
  int        i, j, k, n;
  int        *index_water, *index_oxygen;
  int        n_water_atoms, n_oxygen;
  float      dt, t_old, tsum, tsteps;
  float      d[3], delta_z;
  int        n_diff;
  float      msd;

  /* Flux and Slice location variables */
  int        new_slice, old_slice, comp1, comp2, comp;
  float      fstay, fup, fdown, fnew, fleave;
  float      position;
  float      Constant=1.66033;  /* Atomic mass unit = 1.66033e-27kg, nm = 10e-27 m */
  

  /* Plotting variables */
  float      r, zmin, zmax, dummy;
  
  
  /* The water slices */
  struct water_struct *water;  
  struct slice_struct *slice;
  struct diff_struct *diff; 
  struct temp_struct *temp; 


  /* DEBUGGING AND ACCOUNTING */
  int        sum, memory;
  float      fsum; 
  int        flag;
  int        stay, up, down, new, leave;

#define NLEG asize(leg) 
  t_filenm fnm[] = { 
    { efTRX,  "-f", NULL, ffREAD }, 
    { efTPS,  NULL, NULL, ffREAD }, 
    { efXVG,  NULL, "diff_coef_xyz", ffWRITE }, 
    { efXVG, "-p", "diff_coef_xy", ffOPTWR }, 
    { efXVG, "-z", "diff_coef_z", ffOPTWR },
    { efXVG, "-m", "mean_free_paths", ffOPTWR }, 
    { efXVG, "-c", "number_of_coef", ffOPTWR }, 
    { efXVG, "-b", "box_size", ffOPTWR }, 
    { efXVG, "-d", "density", ffOPTWR }, 
    { efXVG, "-l", "flux", ffOPTWR }, 
    { efXVG, "-u", "mov_summary", ffOPTWR }, 
    { efXVG, "-t", "time", ffOPTWR }, 
    { efNDX,  NULL, NULL, ffOPTRD } 
  }; 

#define NFILE asize(fnm) 

  static char *bugs[] = {
    "Results can depend sensitively on the parameter tmin",
    "If there is too little water in a slice the program produces NaN",
    "-tmin takes a number of time FRAMES as argument, not a time in ps"
  };

  /* DEBUGGING */
  debugfile = fopen( "datafile.dat", "w+" );
  if (debugfile == NULL) { fprintf(stdout,"Cannot create datafile"); exit(1); }
  fprintf(debugfile," Slice   Steps    To      t      MSD      D(XYZ)\n");
  
  
  /* GROMACS LOADER */
  CopyRight(stderr,argv[0]); 

#if GMX_VERSION == 3
  parse_common_args(&argc,argv,PCA_CAN_TIME | PCA_CAN_VIEW,
		    NFILE,fnm,asize(pa),pa,asize(desc),desc,asize(bugs),bugs);
#elif GMX_VERSION == 2
  parse_common_args(&argc,argv,PCA_CAN_TIME | PCA_CAN_VIEW,TRUE,
		    NFILE,fnm,asize(pa),pa,asize(desc),desc,asize(bugs),bugs); 
#else
#error Error: Specify GMX_VERSION
#endif
  fprintf (stderr, "\nVersion: %s\n\n", SRCID_g_water_diffusion_c);

  read_tps_conf(ftp2fn(efTPS,NFILE,fnm),title,&top,&x_new,NULL,box,TRUE);
  natoms=read_first_x(&status,ftp2fn(efTRX,NFILE,fnm),&t,&x_new,box); 
  fprintf(stdout,"Number of residues = %d\n", top.atoms.nres); fflush(stdout);

  
  
  /*** Allocate memory ***/
  atom_name    = allocate_matrix(10, natoms);
  residue_name = allocate_matrix(10, natoms);
  if ((atom_number = (int *) malloc(natoms*sizeof(int))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}
  if ((residue_number = (int *) malloc(natoms*sizeof(int))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}
  if ((q = (float *) malloc(natoms*sizeof(float))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}
  if ((m = (float *) malloc(natoms*sizeof(float))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}

  if ((index_water = (int *) malloc(natoms*sizeof(int))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}
  memory = natoms*sizeof(int);
  if ((index_oxygen = (int *) malloc(natoms*sizeof(int))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}
  memory = natoms*sizeof(int);
  snew(x_old, natoms);     




  /* Copy atoms to arrays */
  for (i=0; i<natoms; i++)
      {
        strcpy(residue_name[i], *top.atoms.resname[top.atoms.atom[i].resnr]); 
        strcpy(atom_name[i]   , *top.atoms.atomname[i]);
        atom_number[i]     = i;
        residue_number[i]  = top.atoms.atom[i].resnr;
        m[i]               = top.atoms.atom[i].m;
        q[i]               = top.atoms.atom[i].q;
        x_old[i][0]        = x_new[i][0];
        x_old[i][1]        = x_new[i][1];
        x_old[i][2]        = x_new[i][2];
        }



  /* Finding water atoms and water oxygen atoms */
  fprintf(stdout, "\n\nFinding all water atoms:\n"); fflush(stdout);
  for (i=0, j=0, k=0; i<natoms; i++)
      {
        if ( !strcmp("SOL", *top.atoms.resname[top.atoms.atom[i].resnr]) )
           { 
             index_water[j] = i;              /* All atoms of the water molecules */
             j++;

             if( !strcmp("OW", *top.atoms.atomname[i]) )
               {                 
                index_oxygen[k] = i;          /* Water Oxygen atoms only          */
                k++; 
                }
             }
        }
  n_water_atoms = j;
  n_oxygen      = k;
  fprintf(stdout, "\n\nFound %d water molecules containing %d atoms.\n", n_oxygen, n_water_atoms);
  fprintf(stdout,"Number of slices = %d\n\n", n_slices); fflush(stdout);
  
        

  /* Allocating Memory for program variables */
  if ((diff = (struct diff_struct *) malloc(No_of_diff*sizeof(struct diff_struct))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}
  memory += No_of_diff*sizeof(struct diff_struct);   
  if ((water = (struct water_struct *) malloc(n_oxygen*sizeof(struct water_struct))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}
  memory += n_oxygen*sizeof(struct water_struct);
  if ((slice = (struct slice_struct *) malloc(n_slices*sizeof(struct slice_struct))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}
  memory += n_slices*sizeof(struct slice_struct);
  if ((temp = (struct temp_struct *) malloc(n_slices*sizeof(struct temp_struct))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}
  memory += n_slices*sizeof(struct temp_struct);   
  fprintf(stdout, "TOTAL MEMORY ALLOCATED: %d [Bytes]\n\n", memory);
  
  
    
  /* Initialise output files */  
  out_diff1 = xvgropen_xydy(ftp2fn(efXVG,NFILE,fnm), "Diffusion Coefficients XYZ","Z-axis [nm]","Diffusion Coefficient [nm\\S2\\Nps\\S-1\\N]");       
  out_diff2 = xvgropen_xydy(opt2fn("-p" ,NFILE,fnm), "Diffusion Coefficients XY","Z-axis [nm]","Diffusion Coefficient [nm\\S2\\Nps\\S-1\\N]");
  out_diff3 = xvgropen_xydy(opt2fn("-z" ,NFILE,fnm), "Diffusion Coefficients Z","Z-axis [nm]","Diffusion Coefficient [nm\\S2\\Nps\\S-1\\N]");
  out_dens  = xvgropen_xydy(opt2fn("-d" ,NFILE,fnm), "Density of Water","Z axis [nm]","Density [g dm\\S-3\\N]");
  out_mean  = xvgropen_xydy(opt2fn("-m" ,NFILE,fnm), "Average Mean Free Path","Z-axis [nm]","Mean Path [nm]");
  out_box   = xvgropen_nxy( opt2fn("-b" ,NFILE,fnm), "Box size","Time [ps]","X,Y,Z-axis [nm]");
  out_numb  = xvgropen_nxy( opt2fn("-c" ,NFILE,fnm), "Coefficients","Z axis [nm]","Number of Coefficients");
  out_flux  = xvgropen_nxy( opt2fn("-l" ,NFILE,fnm), "Flux of Water Molecules","Z axis [nm]","Flux [N nm\\S2\\Nps\\S-1\\N]");
  out_sum   = xvgropen_nxy( opt2fn("-u" ,NFILE,fnm), "Average water movement","Z axis [nm]","Number of Waters");
  out_time  = xvgropen_nxy( opt2fn("-t" ,NFILE,fnm), "Average Residency Time","Z axis [nm]","Time [ps]");
  xvgr_legend(out_flux,2,lega);
  xvgr_legend(out_sum,4,legb);
  fprintf(out_sum,"@ subtitle \"averaged over all timeframes\"\n"); 
 

  out_maple = create_maple_output("maple_density.txt");


  /* Initialisations */  
  tsteps = 0.0;    /* The number of timesteps           */
  tsum   = 0.0;    /* Total time of trajectory          */
  t_old  = t;      /* Initial time                      */
  n_diff = 0;      /* Number of Diff coeff              */
  area   = 0;      /* Cross sectional area (x*y)        */
  flag   = 0;      /* This is 0 in the first step ONLY  */
 
  for(j=0; j<n_slices; j++)
     { 
       slice[j].n_stay = 0;
       slice[j].n_new  = 0;
       slice[j].n_old  = 0;
       temp[j].n_stay  = 0;
       temp[j].n_new   = 0;
       temp[j].n_old   = 0;
       temp[j].n_up    = 0;
       temp[j].n_down  = 0;
       temp[j].mass    = 0.0;
       temp[j].charge  = 0.0;
       temp[j].atoms   = 0.0;
       slice[j].density_M         = 0.0;
       slice[j].density_S         = 0.0;
       slice[j].diff_xyz_M        = 0.0;
       slice[j].diff_xyz_S        = 0.0;
       slice[j].diff_xy_M         = 0.0;
       slice[j].diff_xy_S         = 0.0;
       slice[j].diff_z_M          = 0.0;
       slice[j].diff_z_S          = 0.0;
       slice[j].mean_free_path_M  = 0.0;
       slice[j].mean_free_path_S  = 0.0;
       } 
  
  for(i=0; i<n_oxygen; i++)
     {
       water[i].slice  = -9999;   /* This slice must not exist */
       water[i].nsteps = 0.0;
       }
  


  /**************************************/
  /******  MAIN LOOP STARTS HERE  *******/
  /**************************************/  
  
  do {
       dt      = t - t_old;    /* Timestep          */   
       t_old   = t;            /* time of last step */
       tsum   += dt;           /* total time so far */
       tsteps += 1.0;          /* number of frames  */    

      
       /* Box size, area and delta_z */
       Box[0]   = box[XX][XX];      
       Box[1]   = box[YY][YY];             
       Box[2]   = box[ZZ][ZZ];             
       area_tmp = Box[0]*Box[1];
       area    += area_tmp;
       delta_z  = Box[2]/((float) n_slices);
       fprintf(out_box, "%f\t%f\t%f\t%f\n", t, Box[0], Box[1], Box[2]);                                 
             
             
       for(i=0; i<n_oxygen; i++)
          {      
            old_slice = water[i].slice;            
            
            /* Locate the slice of the current water molecule */
            for (new_slice=-1, position=0.0; x_new[index_oxygen[i]][2]>position && new_slice<n_slices; new_slice++){ position += delta_z; } 
             
            /* Checking outside the bounds just in case: */
            if( new_slice < 0 ){ new_slice = 0; }
            if( new_slice > n_slices-1 ){ new_slice = n_slices-1; }
          
            
            if( new_slice == old_slice)       /* Water i was in the same slice in the last timestep! */
              {
                get_mean_free_path(x_new, x_old, index_oxygen[i], Box, d);

                water[i].nsteps     += 1.0;
                water[i].t_current  += dt;
                water[i].ax         += d[0];
                water[i].ay         += d[1];
                water[i].az         += d[2];
                water[i].mod_a      += sqrt(d[0]*d[0] + d[1]*d[1] + d[2]*d[2]);
                water[i].a2x        += d[0]*d[0];
                water[i].a2y        += d[1]*d[1];
                water[i].a2z        += d[2]*d[2];

                temp[new_slice].n_stay     += 1;
                }
            else                              /* Water i is new in this slice!! */                      
              {
                                                
                if (water[i].nsteps >= t_min) /* Calculate diffusion coefficient of water and add it to the slice */
                   {
                     /* Use the old entries to calculate the diffusion coefficient       */
                     /* the water has to stay at least t_min steps in the slice          */
                     /* Counts how many coefficients have been calculated for this slice */
                     slice[old_slice].n_diff   += 1;  
                     
                     diff[n_diff].slice      = water[i].slice;                                          
                     diff[n_diff].nsteps     = water[i].nsteps;   
                     diff[n_diff].t_origin   = water[i].t_origin;
                     diff[n_diff].mean_ax    = water[i].ax / water[i].nsteps;                   
                     diff[n_diff].mean_ay    = water[i].ay / water[i].nsteps;
                     diff[n_diff].mean_az    = water[i].az / water[i].nsteps;
                     diff[n_diff].mean_mod_a = water[i].mod_a / water[i].nsteps;
                     diff[n_diff].mean_a2x   = water[i].a2x / water[i].nsteps;                    
                     diff[n_diff].mean_a2y   = water[i].a2y / water[i].nsteps;
                     diff[n_diff].mean_a2z   = water[i].a2z / water[i].nsteps;                                            
	             diff[n_diff].diff_z     = diff[n_diff].mean_a2z;                   
	             diff[n_diff].diff_z    /= 2.0 * dt;                   
	             diff[n_diff].diff_xy    = diff[n_diff].mean_a2x + diff[n_diff].mean_a2y;                   
	             diff[n_diff].diff_xy   /= 4.0 * dt;
                     msd                     = diff[n_diff].mean_a2x + diff[n_diff].mean_a2y + diff[n_diff].mean_a2z;
                     diff[n_diff].diff_xyz   = msd / (6.0 * dt);
                     fprintf(debugfile, "%6d   %4.0f  %5.0f  %5.0f  %9.2E  %9.2E\n", 
                                       diff[n_diff].slice, diff[n_diff].nsteps,
                                       diff[n_diff].t_origin, t, msd, 
                                       diff[n_diff].diff_xyz);                          
                     n_diff++;
                     
                     /* End program if No_of_diff diff. coeff. have been calculated */
                     if(n_diff >= No_of_diff){ goto end_of_loop; }
                     }                
                
                if (flag == 1) /* THIS CHECKS FOR THE FIRST FRAME */
                   {     
                     /* How long has the water molecule stayed in the slice */
                     slice[old_slice].delta_t += water[i].t_current - water[i].t_origin;
                     slice[old_slice].n_times += 1.0;
                                
                                
                     /* Calculate the up/down flux */
                     /* comp1 is the distance over the edges of the box */      
                     if(new_slice < old_slice) 
                         {  comp1 = new_slice + n_slices - old_slice; }
                     else{  comp1 = new_slice - n_slices - old_slice; }
                     
                     comp2 = new_slice - old_slice;                  /* comp2 is the distance within the box */
                                          
                     if( abs(comp1) > fabs(comp2) ){ comp = comp2; } /* Checking whether the atom moves over the box boundary */ 
                     else{                           comp = comp1; } /* Always take the shortest distance between two steps   */ 
                     
                     if(comp > 0) { temp[old_slice].n_up   += 1; }
                     else{          temp[old_slice].n_down += 1; }
                     }

                
                /* Create a new entry, water molecule has moved into a new slice */
                water[i].slice         = new_slice;
                water[i].nsteps        = 0.0;
                water[i].t_origin      = t;
                water[i].t_current     = t;
                water[i].ax            = 0.0;
                water[i].ay            = 0.0;
                water[i].az            = 0.0;
                water[i].mod_a         = 0.0;
                water[i].a2x           = 0.0;
                water[i].a2y           = 0.0;
                water[i].a2z           = 0.0;

                temp[new_slice].n_new += 1;
                }  
  }

  
  /*** DEBUGGING ***/  
  /*    
  fprintf(stdout, "\n");
  sum = new = stay = leave = up = down = 0;
  for(j=n_slices-1; j>-1; j--)     
     { 
       fprintf(stdout, "Slice %2d: Staying = %4d  Leaving = %4d  New = %4d  Up = %3d  Down = %3d   Total = %4d\n", 
                       j, temp[j].n_stay, temp[j].n_old-temp[j].n_stay, temp[j].n_new, temp[j].n_up, temp[j].n_down, temp[j].n_new + temp[j].n_stay);
       up    += temp[j].n_up;
       down  += temp[j].n_down;       
       new   += temp[j].n_new;       
       stay  += temp[j].n_stay;       
       leave += temp[j].n_old - temp[j].n_stay;
       sum   += temp[j].n_new + temp[j].n_stay;
       }
  fprintf(stdout, "Sum over all %3d slices = %4d  (= %4d)\n\n", n_slices, sum, n_oxygen);
  fprintf(stdout, "SUMMARY:   Staying = %4d  Leaving = %4d  New = %4d  Up = %3d  Down = %3d   Total = %4d\n",stay,leave,new,up,down,sum);
  fprintf(stdout, "\nLeaving = %5d  New = %5d   (Up+Down = %5d)\n", leave, new, up+down);
  */
  
  
  
  if (flag == 1) /* THIS CHECKS FOR THE FIRST FRAME */
     {        
       /* FLUX CALCULATION */         
       for(j=0; j<n_slices; j++)
          { 
            slice[j].n_old     += ((float) temp[j].n_old);
            temp[j].n_old       = temp[j].n_new + temp[j].n_stay;       
            slice[j].n_stay    += ((float) temp[j].n_stay);
            slice[j].n_new     += ((float) temp[j].n_new);
            slice[j].flux_up   += ((float) temp[j].n_up);     
            slice[j].flux_down += ((float) temp[j].n_down); 
            temp[j].n_stay      = 0;
            temp[j].n_new       = 0;
            temp[j].n_up        = 0;
            temp[j].n_down      = 0;
            }
       }
   else{ 
       for(j=0; j<n_slices; j++)
          { 
            temp[j].n_old       = temp[j].n_new; 
            temp[j].n_stay      = 0;
            temp[j].n_new       = 0;
            temp[j].n_up        = 0;
            temp[j].n_down      = 0;            
            }
        }          
   flag = 1;
          
   
   
   /* DENSITY CALCULATION (looking at all atoms) */
   for(i=0; i<n_water_atoms; i++)
      {      
        for(new_slice=-1, position=0.0; x_new[index_water[i]][2]>position && new_slice<n_slices; new_slice++){ position += delta_z; } 
         
        if(new_slice < 0){ new_slice = 0; }
        if(new_slice > n_slices-1){ new_slice = n_slices-1; }
        
        temp[new_slice].mass   += m[index_water[i]];
        temp[new_slice].charge += q[index_water[i]];
        temp[new_slice].atoms  += 1.0;
        }


   /* 3D density graphs (Maple format), write out only every "maple"-th frame */     
   if(!( ((int) tsteps-1 ) % maple))
     {  out_maple = advance_maple_output(out_maple, 0.0, delta_z, t, n_slices, temp, Constant/(area_tmp*delta_z) ); }



   for(j=0; j<n_slices; j++)
      { 
       slice[j].density_M += temp[j].mass;
       slice[j].density_S += temp[j].mass*temp[j].mass;
       temp[j].mass   = 0.0;
       temp[j].charge = 0.0;
       temp[j].atoms  = 0.0;
       }
 
   /* save old coordinates for next step */
   for(i=0; i<natoms; i++)
      {
       x_old[i][0] = x_new[i][0];
       x_old[i][1] = x_new[i][1];
       x_old[i][2] = x_new[i][2];
       }

   if(!( ((int) tsteps )% t_min)){ fprintf(stdout,"-->  Number of Coefficients = %d\n", n_diff); fflush(stdout); }
   
   } while(read_next_x(status,&t,natoms,x_new,box));
  
   /**********************************/
   /****** MAIN LOOP ENDS HERE *******/
   /**********************************/  



  tsteps += 1.0;


  end_of_loop:
  
  tsteps -= 1.0;     
  tsum   /= tsteps;       
  area   /= (tsteps+1.0);
  Constant /= (area*delta_z);            /* getting the correct units */
  fprintf(stdout,"\n\nMean dt = %.3f,  Number of steps = %.3f\n", tsum, tsteps); 
  fprintf(stdout,"Mean box cross-section (x,y) = %.3f nm (should be ~ %.3f nm)\n", area, Box[0]*Box[1]);
  fprintf(stdout,"Mean slice volume = %.3f nm  (slice height = %.3f nm)\n", area*delta_z, delta_z);


        
  for(j=0; j<n_slices; j++)
     { 
       slice[j].n_old     /= (tsteps - 1.0);  /* Because in the 1st step n_old = 0! */
       slice[j].n_stay    /= (tsteps - 1.0);
       slice[j].n_new     /= (tsteps - 1.0);
       slice[j].flux_up   /= ((tsteps - 1.0) * area * tsum); 
       slice[j].flux_down /= ((tsteps - 1.0) * area * tsum);
       }


  /*** OVERALL SUMMARY OF MOTION ***/
  fprintf(stdout, "\n");   
  fprintf(stdout, "Slice    Staying    Leaving       New       Up       Down      Total\n");
  fprintf(stdout, "---------------------------------------------------------------------\n");
  
  fsum = fnew = fstay = fleave = fup = fdown = 0.0;
  for(j=n_slices-1; j>-1; j--)     
     { 
       fprintf(stdout, " %3d     %7.2f    %7.2f    %7.2f    %6.2f    %6.2f   %8.2f\n",
                       j, slice[j].n_stay, slice[j].n_old-slice[j].n_stay, slice[j].n_new, slice[j].flux_up*area*tsum, slice[j].flux_down*area*tsum, slice[j].n_new + slice[j].n_stay);       
       fup    += slice[j].flux_up   * area * tsum;
       fdown  += slice[j].flux_down * area * tsum;       
       fnew   += slice[j].n_new;       
       fstay  += slice[j].n_stay;       
       fleave += slice[j].n_old - slice[j].n_stay;
       fsum   += slice[j].n_new + slice[j].n_stay;
       }
  fprintf(stdout, "---------------------------------------------------------------------\n");
  fprintf(stdout, "SUMMARY %8.2f   %8.2f   %8.2f   %7.2f   %7.2f  %9.2f\n",fstay,fleave,fnew,fup,fdown,fsum);
  fprintf(stdout, "---------------------------------------------------------------------\n");
  fflush(stdout);




  /* Calculating Means (mean = <x>) and STDEV (stdev = <x> - <x>) */
  for(i=0; i<n_diff; i++)
     {
       for(j=0; j<n_slices; j++)
          {
            if(diff[i].slice == j)
              { 
                r = sqrt(   diff[i].mean_ax * diff[i].mean_ax 
                          + diff[i].mean_ay * diff[i].mean_ay 
                          + diff[i].mean_az * diff[i].mean_az ); 
                
                slice[j].mean_free_path_M += r;
                slice[j].mean_free_path_S += r*r;                
                slice[j].diff_xyz_M       += diff[i].diff_xyz;
                slice[j].diff_xyz_S       += diff[i].diff_xyz*diff[i].diff_xyz;
                slice[j].diff_xy_M        += diff[i].diff_xy;
                slice[j].diff_xy_S        += diff[i].diff_xy*diff[i].diff_xy;
                slice[j].diff_z_M         += diff[i].diff_z;
                slice[j].diff_z_S         += diff[i].diff_z*diff[i].diff_z;
                break;
                }}
       }



  for(j=0, dummy=0.0; j<n_slices; j++, dummy+=1.0)
     {
       
       if (slice[j].n_times > 0){ slice[j].delta_t /= slice[j].n_times; }

       if (slice[j].density_M > 0)
          {                         
            slice[j].density_M /= tsteps; 
            slice[j].density_S /= tsteps;  
            slice[j].density_S -= slice[j].density_M*slice[j].density_M; 
            slice[j].density_S  = sqrt(fabs(slice[j].density_S));            
            }
        
       if (slice[j].n_diff != 0)
          {
            slice[j].mean_free_path_M /= slice[j].n_diff;
            slice[j].diff_xyz_M       /= slice[j].n_diff;
            slice[j].diff_xy_M        /= slice[j].n_diff;
            slice[j].diff_z_M         /= slice[j].n_diff;
            slice[j].mean_free_path_S /= slice[j].n_diff;
            slice[j].mean_free_path_S -= slice[j].mean_free_path_M*slice[j].mean_free_path_M;
            slice[j].mean_free_path_S  = sqrt(fabs(slice[j].mean_free_path_S));       
            slice[j].diff_xyz_S       /= slice[j].n_diff;
            slice[j].diff_xyz_S       -= slice[j].diff_xyz_M*slice[j].diff_xyz_M;
            slice[j].diff_xyz_S        = sqrt(fabs(slice[j].diff_xyz_S));
            slice[j].diff_xy_S        /= slice[j].n_diff;
            slice[j].diff_xy_S        -= slice[j].diff_xy_M*slice[j].diff_xy_M;
            slice[j].diff_xy_S         = sqrt(fabs(slice[j].diff_xy_S));
            slice[j].diff_z_S         /= slice[j].n_diff;
            slice[j].diff_z_S         -= slice[j].diff_z_M*slice[j].diff_z_M;
            slice[j].diff_z_S          = sqrt(fabs(slice[j].diff_z_S));
            }

  
       zmin = dummy * delta_z;        /* Bottom of slice j */
       zmax = zmin + (0.5*delta_z);   /* Centre of slice j */
     
       fprintf(out_diff1,"%f\t%E\t%E\n", zmax, slice[j].diff_xyz_M, slice[j].diff_xyz_S);     /* Diffusion coefficients */
       fprintf(out_diff2,"%f\t%E\t%E\n", zmax, slice[j].diff_xy_M,  slice[j].diff_xy_S);      /* Diffusion coefficients */
       fprintf(out_diff3,"%f\t%E\t%E\n", zmax, slice[j].diff_z_M,   slice[j].diff_z_S);       /* Diffusion coefficients */
       fprintf(out_mean,"%f\t%E\t%E\n",  zmax, slice[j].mean_free_path_M, slice[j].mean_free_path_S);   /* Mean Free Paths        */ 
       fprintf(out_numb,"%f\t%d\n",      zmax, slice[j].n_diff);                              /* Number of Coefficients */
       fprintf(out_dens,"%f\t%E\t%E\n",  zmax, slice[j].density_M*Constant,  slice[j].density_S*Constant);      /* Density                */ 
       fprintf(out_flux,"%f\t%E\t%E\n",  zmax, slice[j].flux_up,    slice[j].flux_down);      /* Flux                   */ 
       fprintf(out_sum ,"%f\t%E\t%E\t%E\t%E\n", zmax, slice[j].n_stay, slice[j].n_old - slice[j].n_stay, slice[j].n_new, slice[j].n_new + slice[j].n_stay);  /* Movement summary       */ 
       fprintf(out_time,"%f\t%E\n",      zmax, slice[j].delta_t);       /* Average residence time of a water in a slice */ 
       }


  close_maple_output(out_maple);

  
    
  close_trj(status);
  
  fclose(out_box);
  fclose(out_sum);
  fclose(out_dens);
  fclose(out_flux);
  fclose(out_mean);
  fclose(out_numb);
  fclose(out_time);
  fclose(out_diff1);
  fclose(out_diff2);
  fclose(out_diff3);
  fclose(debugfile);
     
  return 0;
}







        
/**** FUNCTIONS ****/


void get_mean_free_path(rvec *x_new, rvec *x_old, int index, float *Box, float *d)
{
  int j; /* j loops over the 3 spatial dimensions */
  float d1[3], d2[3];      

  for(j=0; j<3; j++)
     { 
       /* d1 is the distance over the edges of the box */      
       if(x_new[index][j] < x_old[index][j]) 
         {    d1[j] = x_new[index][j] + Box[j] - x_old[index][j]; }
       else{  d1[j] = x_new[index][j] - Box[j] - x_old[index][j]; }
       
       /* d2 is the distance within the box */
       d2[j] = x_new[index][j] - x_old[index][j];
       
       /* Checking whether the atom moves over the box boundary */
       /* Always take the shortest distance between two steps   */
       if( fabs(d1[j]) > fabs(d2[j]) ){ d[j] = d2[j]; }
       else{                            d[j] = d1[j]; }
       }
}

        

        
char ** allocate_matrix(int length_of_element, int number_of_elements)
{
  /* Adapted from NUMERICAL RECIPES (Press et al) */
  
  int i;
  char **m;
  
  length_of_element++;
  number_of_elements++;
  
  /* Allocate memory for pointers to elements */
  if ((m = (char **) malloc(number_of_elements*sizeof(char*))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}

  /* Allocate memory for the elements */
  if ((m[0] = (char *) malloc(number_of_elements*length_of_element*sizeof(char))) == NULL)
     { fprintf(stdout, "Not enough memory to allocate buffer!\n"); exit(1);}

  /* Set pointers to elements */
  for(i=1; i<number_of_elements+1; i++){ m[i] = m[i-1] + length_of_element; }
  
  /* Return the pointer */
  return m;  
} 




/* This type supports multiple columns. Format: x y1 y2 ... */
FILE *xvgropen_nxy(char *fn,char *title,char *xaxis,char *yaxis)
{
  FILE *xvgr;
  time_t t;
  
  xvgr=(FILE *)ffopen(fn,"w");
  time(&t);
  fprintf(xvgr,"# File created: %s",ctime(&t));
  fprintf(xvgr,"#\n");
  fprintf(xvgr,"@    title \"%s\"\n",title);
  fprintf(xvgr,"@    xaxis  label \"%s\"\n",xaxis);
  fprintf(xvgr,"@    yaxis  label \"%s\"\n",yaxis);
  fprintf(xvgr,"@TYPE nxy\n");
  
  return xvgr;
}




/* This type supports standard deviations. Format: x y stdev(y) */
FILE *xvgropen_xydy(char *fn,char *title,char *xaxis,char *yaxis)
{
  FILE *xvgr;
  time_t t;
  
  xvgr=(FILE *)ffopen(fn,"w");
  time(&t);
  fprintf(xvgr,"# File created: %s",ctime(&t));
  fprintf(xvgr,"#\n");
  fprintf(xvgr,"@    title \"%s\"\n",title);
  fprintf(xvgr,"@    xaxis  label \"%s\"\n",xaxis);
  fprintf(xvgr,"@    yaxis  label \"%s\"\n",yaxis);
  fprintf(xvgr,"@    s0 symbol 3\n");                       /* Black filled squares  */
  fprintf(xvgr,"@    s0 symbol size 0.600000\n");
  fprintf(xvgr,"@    s0 symbol fill 1\n");
  fprintf(xvgr,"@    s0 symbol color 2\n");
  fprintf(xvgr,"@    s0 errorbar type BOTH\n");             /* Error bars are dotted */
  fprintf(xvgr,"@    s0 errorbar length 0.600000\n");
  fprintf(xvgr,"@    s0 errorbar linewidth 1\n");
  fprintf(xvgr,"@    s0 errorbar linestyle 2\n");
  fprintf(xvgr,"@    s0 errorbar riser on\n");
  fprintf(xvgr,"@    s0 errorbar riser linewidth 1\n");
  fprintf(xvgr,"@    s0 errorbar riser linestyle 2\n");
  fprintf(xvgr,"@TYPE xydy\n");

  return xvgr;
}

FILE *create_maple_output(char *filename)
{

 FILE *f_out;       
 
 /* Opening files */
 f_out = fopen( filename, "w+" );
 if(f_out == NULL){ fprintf(stdout, "Cannot create file: %s.\n", filename); exit(1); } 
 fprintf(f_out,"data := [["); 

 return f_out;
    
}



FILE *advance_maple_output(FILE *f_out, float xmin, float dx, float y, int N, struct temp_struct *temp, float K )
{
 int i;
 static int j=0;
 float x;
 
 fprintf(stdout,"-->  Adding a contour to the surface plot.\n");

 if(j>0){ fprintf(f_out,"],\n["); }
 for(i=0, x=xmin; i<N; i++, x+=dx)
    {
/*      fprintf(f_out,"[%.4f,%.4f,%.4f]", (float) i, (float) j, temp[i].mass*K); THIS WORKS */
      fprintf(f_out,"[%.4f,%.4f,%.4f]", x, y, temp[i].mass*K); 
      if(i<N-1){fprintf(f_out,","); }
      }
 j++;
 return f_out;
}    



void close_maple_output(FILE *f_out)
{
  fprintf(stdout,"Finishing the surface plot.\n\n");
  fprintf(f_out,"]]:\n");
  fprintf(f_out,"with(plots):\n");
  fprintf(f_out,"surfdata( data, axes=frame, scaling=UNCONSTRAINED, labeldirections=[HORIZONTAL,HORIZONTAL,VERTICAL],");
  fprintf(f_out," labels=[\"Z-axis [nm]\",\"Time [ps]\",\"Density [g/dm]\"], title=\"Density of Water\" );");     
  fclose(f_out);  
}
